/**
* Contains properties and methods for storing and
* performing calculations on electronic displays.
*/
var ScreenCalc = (function () {
    function ScreenCalc(pixelWidth, pixelHeight, diagonalSize) {
        this.setPixelWidth(pixelWidth);
        this.setPixelHeight(pixelHeight);
        this.setDiagonalSize(diagonalSize);
    }
    ScreenCalc.prototype.getPixelWidth = function () {
        return this.pixelWidth;
    };

    ScreenCalc.prototype.setPixelWidth = function (pixelWidth) {
        if (!ScreenCalc.isPositiveInt(pixelWidth)) {
            throw new Error("pixelWidth must be a positive integer");
        } else {
            this.pixelWidth = pixelWidth;
        }
    };

    ScreenCalc.prototype.getPixelHeight = function () {
        return this.pixelHeight;
    };

    ScreenCalc.prototype.setPixelHeight = function (pixelHeight) {
        if (!ScreenCalc.isPositiveInt(pixelHeight)) {
            throw new Error("pixelHeight must be a positive integer");
        } else {
            this.pixelHeight = pixelHeight;
        }
    };

    /** Returns the diagonal size of the display */
    ScreenCalc.prototype.getDiagonalSize = function () {
        return this.diagonalSize;
    };

    ScreenCalc.prototype.setDiagonalSize = function (diagonalSize) {
        if (!ScreenCalc.isPositiveNum(diagonalSize)) {
            diagonalSize = 0;
        }

        this.diagonalSize = diagonalSize;
    };

    /**
    * Returns the number of pixels per unit in the display
    * (will be ppi if units are in, or ppcm if units are cm)
    */
    ScreenCalc.prototype.getPixelDensity = function () {
        var density = this.pixelWidth / this.getPhysicalWidth();
        return Math.round(density);
    };

    /**
    * Set the display's diagonal size from its pixel density (pixels per unit)
    */
    ScreenCalc.prototype.setPixelDensity = function (ppu) {
        var physicalHeight = this.pixelHeight / ppu;
        var physicalWidth = physicalHeight * this.getRatio();

        // base squared + height squared = diagonal squared
        var diagonalSquared = Math.pow(physicalHeight, 2) + Math.pow(physicalWidth, 2);
        this.diagonalSize = Math.sqrt(diagonalSquared);
    };

    /** Returns the physical width of the display */
    ScreenCalc.prototype.getPhysicalWidth = function () {
        return this.getPhysicalHeight() * this.getRatio();
    };

    /** Returns the physical height of the display */
    ScreenCalc.prototype.getPhysicalHeight = function () {
        /*
        * Pythagorean theorem: base squared + height squared = diagonal size squared.
        * The physical width and height aren't yet known, but the ratio is. The ratio
        * can be used to define the base in terms of height, and solve for height using
        * the diagonal size.
        *
        * height squared + (ratio * height) squared = diagonal squared
        * Expanded: height squared + ratio squared * height squared = diagonal squared
        *
        * Since ratio squared is multiplied by height squared, adding height squared
        * simply increases this multiple by 1. Therefore:
        * (ratio squared + 1) * height squared = diagonal squared
        */
        var ratioSquared = Math.pow(this.getRatio(), 2);
        var diagonalSquared = Math.pow(this.diagonalSize, 2);
        var baseSquared = ratioSquared + 1;
        var heightSquared = diagonalSquared / baseSquared;
        var height = Math.sqrt(heightSquared);
        return height;
    };

    /** Returns the area of the display in square units */
    ScreenCalc.prototype.getArea = function () {
        return this.getPhysicalHeight() * this.getPhysicalWidth();
    };

    /** Returns the total number of pixels in the screen */
    ScreenCalc.prototype.getPixelCount = function () {
        return this.getPixelWidth() * this.getPixelHeight();
    };

    /**
    * Returns the screen's width divided by its height.
    * For example, the ratio of a 1920x1080 display would
    * be 1.78 when rounded to two decimal places.
    */
    ScreenCalc.prototype.getRatio = function () {
        return this.pixelWidth / this.pixelHeight;
    };

    /**
    * Returns a screen's ratio as a simplified string.
    * For example, a 1920x1080 display would return 16:9
    */
    ScreenCalc.prototype.getStringRatio = function () {
        // start by finding the greatest common divisor of the pixel width and pixel height
        var gcd = gcdRecursive(this.pixelWidth, this.pixelHeight);

        // then divide the width and height by the gcd to find the simplest width to height ratio
        var simpleWidth = this.pixelWidth / gcd;
        var simpleHeight = this.pixelHeight / gcd;

        // if the simplified width goes evenly into 16,
        // and the simplified height is not 3, multiply
        // both sides by the quotient so that the ratio is 16:n
        if (simpleHeight !== 3 && (16 % simpleWidth === 0)) {
            var quotient = 16 / simpleWidth;
            simpleWidth = 16;
            simpleHeight = simpleHeight * quotient;
        }

        // return as string
        return simpleWidth.toString() + ':' + simpleHeight.toString();

        /** Returns the greatest common divisor of a and b */
        function gcdRecursive(a, b) {
            if (!b) {
                return a;
            }

            return gcdRecursive(b, a % b);
        }
    };

    /** Returns true if the specified value is a positive integer */
    ScreenCalc.isPositiveInt = function (val) {
        var y = parseInt(val);
        if (isNaN(y)) {
            return false;
        }

        return val === y && val.toString() === y.toString() && val > 0;
    };

    /** Returns true if the value is a positive number */
    ScreenCalc.isPositiveNum = function (val) {
        return typeof val === "number" && val > 0;
    };
    return ScreenCalc;
})();

module.exports = ScreenCalc;
//# sourceMappingURL=ScreenCalc.js.map
